// ----------------------------------------------------------------------------
// * Raima Database Manager
// *
// * Copyright (c) 2012 Raima Inc.,  All rights reserved.
// *
// * Use of this software, whether in source code format, or in executable,
// * binary object code form, is governed by the Raima LICENSE which
// * is fully described in the LICENSE.TXT file, included within this
// * distribution of files.
// * ----------------------------------------------------------------------------

#include <iostream>

// rdm.h is required in all files using the RDM Database
#include "rdm.h"

// obj04_db.h was generated from the DDL file obj04_db.ddl.
#include "obj04_db.h"

// obj04_db_api.h was generated from the DDL file obj04_db.ddl. with 
// the -cpp option
#include "obj04_db_api.h"

// Name space for the RDM C++ interface
using namespace RDM_CPP;

// EXAMPLE - obj04 for C++
// Each time you run this example:
//   1) The database is initialized (all existing contents removed)
//   2) Several new artists are added to the database
//   3) For each artist a couple albums are added to the database
//   4) For each album a tracks are added to the database
//   5) A list of each artist, their albums, and the album's tracks is 
//      displayed

// The DDL (Database Definition Language) is in obj04_db.ddl. The database
// schema is embedded into the application and the database will be created
// the first time the application is run.  Subsequent runs will use the now
// existing database.

// The following RDM C++ methods are illustrated:
//      Db->Open
//      Db->New_recordWithFieldValues
//      Db->Get_records
//      Cursor->AfterLast
//      Cursor->GetFieldValues
//      Cursor->GetMembers
//      Cursor->Connect
//      Cursor++
// They are fully documented in http://docs.raima.com/rdm/.

// Each of these methods can throw and an exception of type rdm_exception

// The actual database is stored in a directory named 'obj04_db' in this
// project directory.

extern const char *the_doors_tracks[11];
extern const char *strange_days_tracks[10];
extern const char *waiting_tracks[11];
extern const char *the_rolling_stones_tracks[12];
extern const char *out_of_our_heads_tracks[12];
extern const char *beggars_banquet_tracks[10];
extern const char *tattoo_you_tracks[9];
extern const char *bleach_tracks[11];
extern const char *nevermind_tracks[13];
extern const char *in_utero_tracks[12];

class obj04
{
public:
    Db_obj04_db   db;
    Cursor_album  album_cursor;
    Cursor_artist artist_cursor;
    Cursor_track  track_cursor;

    void InsertData () throw (rdm_exception&);
    void DisplayCollection () throw (rdm_exception&);
    void Initialize () throw (rdm_exception&);
    void Cleanup () throw (rdm_exception&);
    void AddAlbum (const char *album_title,
                   uint16_t    num_tracks,
                   const char *track_names[]) throw (rdm_exception&);
};

void obj04::AddAlbum(
    const char     *album_title,
    uint16_t        num_tracks,
    const char     *track_names[])
{
    int32_t ii;
    struct album album_rec;
    struct track track_rec;

    strcpy(album_rec.title, album_title);

    try
    {
        // Add the album record
        album_cursor = db.New_album_recordWithFieldValues(&album_rec);

        // Connect it to the current artist
        album_cursor.Connect_artist_album(artist_cursor);

        // Add the tracks for the album
        for (ii = 0; ii < num_tracks; ii++)
        {
            strcpy(track_rec.name, track_names[ii]);

            // Add the track record
            track_cursor = db.New_track_recordWithFieldValues(&track_rec);

            // Associate the track with the album
            track_cursor.Connect_album_track(album_cursor);
        }
    }
    catch (rdm_exception& e)
    {
        cerr << "Error adding an album (" << e.error() << " - " << e.what() << ")" << endl;
        throw (e);
        return;
    }
    return;
}

void obj04::Initialize()
{
    // Open the database and initialize it
    try
    {
        db = db.Open("obj04_db", "x");
        db.Initialize();
    }
    catch (rdm_exception& e)
    {
        cerr << "Error " << e.error() << " (" << e.what() << ") opening/initializing obj04_db database" << endl;
        throw (e);
        return;
    }
    return;
}

void obj04::InsertData()
{
    struct artist artist_rec;

    // Create the artists, albums, and tracks in the database
    try
    {
        strcpy(artist_rec.name, "The Doors");
        artist_cursor = db.New_artist_recordWithFieldValues(&artist_rec);
        AddAlbum("The Doors", RLEN(the_doors_tracks), the_doors_tracks);
        AddAlbum("Strange Days", RLEN(strange_days_tracks), strange_days_tracks);
        AddAlbum("Waiting for the Sun", RLEN(waiting_tracks), waiting_tracks);

        strcpy(artist_rec.name, "The Rolling Stones");
        artist_cursor = db.New_artist_recordWithFieldValues(&artist_rec);
        AddAlbum("The Rolling Stones", RLEN(the_rolling_stones_tracks), the_rolling_stones_tracks);
        AddAlbum("Out of our Heads", RLEN(out_of_our_heads_tracks), out_of_our_heads_tracks);
        AddAlbum("Beggars Banquest", RLEN(beggars_banquet_tracks), beggars_banquet_tracks);
        AddAlbum("Tattoo You", RLEN(tattoo_you_tracks), tattoo_you_tracks);

        strcpy(artist_rec.name, "Nirvana");
        artist_cursor = db.New_artist_recordWithFieldValues(&artist_rec);
        AddAlbum("Bleach", RLEN(bleach_tracks), bleach_tracks);
        AddAlbum("Nevermind", RLEN(nevermind_tracks), nevermind_tracks);
        AddAlbum("In Utero", RLEN(in_utero_tracks), in_utero_tracks);
    }
    catch (rdm_exception& e)
    {
        cerr << "Error " << e.error() << " (" << e.what() << ") adding records" << endl;
        throw (e);
        return;
    }
    return;
}
    
void obj04::DisplayCollection()
{
    struct artist artist_rec;
    struct album  album_rec;
    struct track  track_rec;
    try
    {
        // Loop through each artist record and display the albums they are 
        // associated with
        artist_cursor = db.Get_artist_records();
        while (!artist_cursor.AfterLast())
        {
            artist_cursor.GetFieldValues(&artist_rec);
            cout << endl << artist_rec.name << endl;

            // Get the the albums for this artist
            album_cursor = artist_cursor.Get_artist_album_members();
            while (!album_cursor.AfterLast())
            {
                album_cursor.GetFieldValues(&album_rec);
                cout << '\t' << album_rec.title << endl;

                // Get the tracks for this album
                track_cursor = album_cursor.Get_album_track_members();
                while ( !track_cursor.AfterLast() )
                {
                    track_cursor.GetFieldValues(&track_rec);
                    cout << "\t\t" << track_rec.name << endl;

                    track_cursor++;
                }
                album_cursor++;
            }
            artist_cursor++;
        }
    }
    catch (rdm_exception& e)
    {
        cerr << "Error " << e.error() << " (" << e.what() << ") reading records" << endl;
    }   
}

void obj04::Cleanup()
{
    return;
}

int32_t main(void)
{
    obj04 obj;

    try
    {
        obj.Initialize();
        obj.InsertData();
        obj.DisplayCollection();
        obj.Cleanup();
    }
    catch (rdm_exception& e)
    {
        cerr << "Error " << e.error() << " (" << e.what() << ") reading records" << endl;
        return EXIT_FAILURE;
    }
    
    cout << endl << "Press <Enter> to continue . . .";
    getchar();
    return EXIT_SUCCESS;
}

const char *the_doors_tracks[11] = 
{
    "Break on Through (To the Other Side)",
    "Soul Kitchen" ,
    "The Crystal Ship",
    "Twentieth Century Fox",
    "Alabama Song (Whisky Bar)",
    "Light My Fire",
    "Back Door Man",
    "I Looked at You",
    "End of the Night",
    "Take It as It Comes",
    "The End"
};

const char *strange_days_tracks[10] = 
{
    "Strange Days",
    "You're Lost Little Girl",
    "Love Me Two Times",
    "Unhappy Girl",
    "Horse Latitudes",
    "Moonlight Drive",
    "People Are Strange",
    "My Eyes Have Seen You",
    "I Can't See Your Face in My Mind",
    "When the Music's Over"
};

const char *waiting_tracks[11] = 
{
    "Hello, I Love You",
    "Love Street",
    "Not to Touch the Earth",
    "Summer's Almost Gone",
    "Wintertime Love",
    "The Unknown Soldier",
    "Spanish Caravan",
    "My Wild Love",
    "We Could Be So Good Together",
    "Yes, the River Knows",
    "Five to One"
};

const char *the_rolling_stones_tracks[12] =
{
    "Not Fade Away",
    "Route 66",
    "I Just Want to Make Love to You",
    "Honest I Do",
    "Now I've Got a Witness",
    "Little by Little",
    "I'm a King Bee",
    "Carol",
    "Tell Me",
    "Can I Get a Witness",
    "You Can Make It If You Try",
    "Walking the Dog"
};

const char *out_of_our_heads_tracks[12] =
{
    "She Said Yeah",
    "Mercy, Mercy",
    "Hitch Hike",
    "That's How Strong My Love Is",
    "Good Times",
    "Gotta Get Away",
    "Talkin' Bout You",
    "Cry to Me",
    "Oh, Baby (We Got a Good Thing Going)",
    "Heart of Stone",
    "The Under Assistant West Coast Promotion Man",
    "I'm Free"
};

const char *beggars_banquet_tracks[10] =
{
    "Sympathy for the Devil",
    "No Expectations",
    "Dear Doctor",
    "Parachute Woman",
    "Jig-Saw Puzzle",
    "Street Fighting Man",
    "Prodigal Son",
    "Stray Cat Blues",
    "Factory Girl",
    "Salt of the Earth"
};

const char *tattoo_you_tracks[9] =
{
    "Start Me Up",
    "Hang Fire",
    "Slave",
    "Little T&A",
    "Black Limousine",
    "Neighbours",
    "Worried About You",
    "Tops",
    "Heaven"
};

const char * bleach_tracks[11] =
{
    "Blew",
    "Floyd The Barber",
    "About a Girl",
    "School",
    "Love Buzz",
    "Paper Cuts",
    "Negative Creep",
    "Scoff",
    "Swap Meet",
    "Mr. Moustache",
    "Sifting"
};

const char * nevermind_tracks[13] =
{
    "Smells Like Teen Spirit",
    "In Bloom",
    "Come as You Are",
    "Breed",
    "Lithium",
    "Polly",
    "Territorial Pissings",
    "Drain You",
    "Lounge Act",
    "Stay Away",
    "On a Plain",
    "Something in the Way",
    "Endless, Nameless"
};

const char * in_utero_tracks[12] =
{
    "Serve the Servants",
    "Scentless Apprentice",
    "Heart-Shaped Box",
    "Rape Me",
    "Frances Farmer Will Have Her Revenge on Seattle",
    "Dumb",
    "Very Ape",
    "Milk It",
    "Pennyroyal Tea",
    "Radio Friendly Unit Shifter",
    "tourette's",
    "All Apologies"
};